package com.sillycycle.bagleyd.abacus;

/*
 * @(#)Abacus.java
 *
 * Copyright 1994 - 2021  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
import android.graphics.Canvas;
import android.graphics.Paint;

import com.sillycycle.bagleyd.abacus.model.AbacusFormat;
import com.sillycycle.bagleyd.abacus.model.NumberField;
import com.sillycycle.bagleyd.abacus.view.AbacusGeometry;
import com.sillycycle.bagleyd.abacus.view.AbacusView;
import com.sillycycle.bagleyd.util.OrientDraw;

/**
 * The <code>Abacus</code> class extends the
 * <code>AbacusCanvas</code> for handling the responses to the user
 * for the <code>AbacusApp</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/abacus.html
 */

public class Abacus extends AbacusCanvas {
	//private static final long serialVersionUID = 42L;

	public Abacus(AbacusFormat model, NumberField field, AbacusGeometry geo, Canvas canvas, Paint paint, AbacusView abacusView) {
		super(model, field, geo, canvas, paint, abacusView);
	}

	public void AbacusCon(
			int fg, int bg, int borderc,
			int beadc1, int beadc2, int beadc3, int markerc,
			int railc1, int railc2, int railc3, int railc4,
			int delay, int rails, boolean vertical,
			int colorScheme, boolean slot,
			boolean diamond, int railIndex,
			boolean topOrient, boolean bottomOrient,
			int topNumber, int bottomNumber,
			int topFactor, int bottomFactor,
			int topSpaces, int bottomSpaces,
			int topPiece, int bottomPiece,
			int topPiecePercent, int bottomPiecePercent,
			int shiftPercent, int subdeck, int subbead,
			boolean sign, int decimalPosition,
			boolean group, int groupSize, boolean placeOnRail,
			boolean decimalComma, int base, int subbase,
			int anomaly, int shiftAnomaly,
			int anomalySq, int shiftAnomalySq,
			int displayBase, int pressOffsetY,
			AbacusInterface.RomanNumeralsFormat romanNumerals,
			boolean latin,
			AbacusInterface.RomanMarkersFormat romanMarkers,
			int mode, int submode,
			boolean subdecksSeparated,
			boolean altSubbeadPlacement) {
		firstPaint = true;
		//FIXME
		/*this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addMouseWheelListener(this);
		this.addKeyListener(this);*/
		if (model == null) {
			model = new AbacusFormat(rails);
			field = new NumberField();
			geo = null;
		}
		model.setMode(mode);
		model.setSubmode(submode);
		model.setBase(base);
		model.setSubbase(subbase);
		model.setRails(rails);
		model.setVertical(vertical);
		model.setColorScheme(colorScheme);
		model.setSlot(slot);
		model.setDiamond(diamond);
		model.setRailIndex(railIndex);
		model.setOrientation(AbacusFormat.TOP, topOrient);
		model.setOrientation(AbacusFormat.BOTTOM, bottomOrient);
		model.setNumber(AbacusFormat.TOP, topNumber);
		model.setNumber(AbacusFormat.BOTTOM, bottomNumber);
		model.setFactor(AbacusFormat.TOP, topFactor);
		model.setFactor(AbacusFormat.BOTTOM, bottomFactor);
		model.setSpaces(AbacusFormat.TOP, topSpaces);
		model.setSpaces(AbacusFormat.BOTTOM, bottomSpaces);
		model.setPiece(AbacusFormat.TOP, topPiece);
		model.setPiece(AbacusFormat.BOTTOM, bottomPiece);
		model.setPiecePercent(AbacusFormat.TOP, topPiecePercent);
		model.setPiecePercent(AbacusFormat.BOTTOM, bottomPiecePercent);
		model.setShiftPercent(shiftPercent);
		model.setSubdeck(subdeck);
		model.setSubbead(subbead);
		model.setSign(sign);
		model.setDecimalPosition(decimalPosition);
		model.setGroupSize(groupSize);
		model.setAnomaly(anomaly);
		model.setShiftAnomaly(shiftAnomaly);
		model.setAnomalySq(anomalySq);
		model.setShiftAnomalySq(shiftAnomalySq);
		model.setRomanMarkers(romanMarkers);
		model.setPlaceOnRail(placeOnRail);
		field.setGroup(group);
		field.setDecimalComma(decimalComma);
		field.setDisplayBase(displayBase);
		field.setRomanNumerals(romanNumerals);
		field.setLatin(latin);
		model.setSubdecksSeparated(subdecksSeparated);
		model.setAltSubbeadPlacement(altSubbeadPlacement);
		model.initializeAbacus();
		if (geo == null) {
			geo = new AbacusGeometry(model);
			aDraw = null;
		}
		//FIXME
		/*if (geo.getCoreSize().x != getWidth()
				|| geo.getCoreSize().y != getHeight()) {
			geo.resize(getWidth(), getHeight());
			aDraw = null;
		}
		if (aDraw == null && (getWidth() != 0 || getHeight() != 0)) {
			aDraw = new AbacusDraw(model, geo, this);
		}*/
		vDraw = new OrientDraw(model.getVertical());
		geo.setFrameColor(fg);
		geo.setBackground(bg);
		geo.setForeground(borderc);
		geo.setBorderColor(borderc);
		geo.setRailColor(0, railc1);
		geo.setRailColor(1, railc2);
		geo.setRailColor(2, railc3);
		geo.setRailColor(3, railc4);
		geo.setMarkerColor(markerc);
		geo.setBeadColor(0, beadc1);
		geo.setBeadColor(1, beadc2);
		geo.setBeadColor(2, beadc3);
		geo.setDelay(delay);
		geo.setNumberSlices((delay < 5 * AbacusGeometry.MAX_SLICES) ?
			delay / 5 + 1 : AbacusGeometry.MAX_SLICES);
		geo.setPressOffsetY(pressOffsetY);
		//FIXME abacusView.setCursor(new Cursor(Cursor.HAND_CURSOR));
	}

	public void processKey(char key, boolean control) {
		switch (key) {
		case '>':
		case '.':
			speedUpAbacus();
			break;
		case '<':
		case ',':
			slowDownAbacus();
			break;
		case '@':
			abacusView.toggleSound();
			break;
		case '?':
			abacusView.descriptionHelp();
			break;
		case '!':
			abacusView.featuresHelp();
			break;
		case '^':
			abacusView.referencesHelp();
			break;
		case 'a':
		case 'A':
			abacusView.aboutHelp();
			break;
		case ' ':
			if (demo)
				showMoreAbacus();
			break;
		case 'c':
		case 'C':
			clearAbacus();
			break;
		case '-':
		case '_':
			complementAbacus();
			break;
		case 'u':
		case 'U':
			undoAbacus();
			break;
		case 'r':
		case 'R':
			redoAbacus();
			break;
		case 'i':
		case 'I':
			incrementAbacus();
			break;
		case 'd':
		case 'D':
			decrementAbacus();
			break;
		case 'f':
		case 'F':
			changeFormatAbacus();
			break;
		case 'm':
		case 'M':
			changeMuseumAbacus();
			break;
		case '#':
			togglePlaceOnRailAbacus();
			break;
		case 'v':
		case 'V':
			changeRomanNumeralsAbacus();
			break;
		case '~':
		case '`':
			toggleLatinAbacus();
			break;
		case 'g':
		case 'G':
			toggleGroupingAbacus();
			break;
		case '|':
			toggleVerticalAbacus();
			break;
		case 's':
		case 'S':
			toggleNegativeSignAbacus();
			break;
		case 't':
		case 'T':
			changePiecePercentAbacus();
			break;
		case 'p':
		case 'P':
			changePieceAbacus();
			break;
		case 'b':
		case 'B':
			changeSubdecksAbacus();
			break;
		case 'z':
		case 'Z':
			changeRomanMarkersAbacus();
			break;
		case 'k':
		case 'K':
			toggleSubdecksSeparatedAbacus();
			break;
		case ')':
			toggleAltSubbeadPlacementAbacus();
			break;
		case 'y':
		case 'Y':
			toggleAnomalyAbacus();
			break;
		case 'o':
		case 'O':
			toggleDemoAbacus();
			break;
		case '$':
			toggleTeachAbacus();
			break;
		case '+':
			toggleRightToLeftAbacusAdd();
			break;
		case '*':
			toggleRightToLeftAbacusMult();
			break;
		case 'n':
		case 'N':
			if (demo)
				showNextAbacus();
			break;
		case 'e':
		case 'E':
			if (demo)
				showEnterAbacus();
			break;
		case 'j':
		case 'J':
			if (demo)
				showJumpAbacus();
			break;
		/*case '0':*/
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		/*case '7':*/
		case '8':
		/*case '9':*/
			if (demo && key >= '1' && key <= '5') {
				showChapterAbacus(key - '1');
			} else if (!demo && (key == 2 || key == 4 || key == 6 || key == 8)) {
				moveAbacusInput(lastX, lastY, key, control);
			}
			break;
		case 'q':
		case 'Q':
		case 'x':
		case 'X':
		//case '\03': /* Control-C */
			if (demo) {
				toggleDemoAbacus();
				break;
			}
			abacusView.quit();
			break;
		default:
		}
	}

	/*FIXME public void mousePressed(MouseEvent event) {
		int modifiers = event.getModifiersEx();

		if (mouseDown)
			return;
		// would need to reset Teach but modal
		if (demo) {
			abacusDemo.moreDemo();
			return;
		}
		if (((modifiers & InputEvent.META_DOWN_MASK) == 0) &&
				((modifiers & InputEvent.ALT_DOWN_MASK) == 0)) {
			selectAbacus(event.getX(), event.getY());
			mouseDown = true;
		} else if (modifiers == InputEvent.BUTTON3_DOWN_MASK) {
			if ((event.getClickCount() & 1) == 1) {
				clearWithQueryAbacus();
			} else {
				clearAbacus();
			}
		}
	}*/

	public void mouseClicked(/*FIXME MouseEvent event*/) {
		/* Method required but mousePressed does work */
	}

	public void mouseReleased(/*FIXME MouseEvent event*/) {
		/*if (mouseDown) { // Do not care which mouse button
			releaseAbacus(event.getX(), event.getY());
			mouseDown = false;
		}*/
	}

	/*FIXME public void mouseEntered(MouseEvent event) {
		if (geo == null)
			return;
		geo.setForeground(geo.getFrameColor());
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	public void mouseExited(MouseEvent event) {
		if (mouseDown && currentDeck >= 0) {
			currentSpace = -1;
			releasePaint = true;
		}
		geo.setForeground(geo.getBorderColor());
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	public void mouseDragged(MouseEvent event) {
		lastX = event.getX();
		lastY = event.getY();
	}

	public void mouseMoved(MouseEvent event) {
		mouseDragged(event);
	}

	public void mouseWheelMoved(MouseWheelEvent event) {
		int notches = event.getWheelRotation();
		int modifiers = event.getModifiersEx();

		if (((modifiers & InputEvent.META_DOWN_MASK) == 0) &&
				((modifiers & InputEvent.ALT_DOWN_MASK) == 0)) {
			wheelAbacus(notches, event.getX(), event.getY());
		}
	}

	public void keyPressed(KeyEvent event) {
		int modifiers = event.getModifiersEx();
		boolean control = ((modifiers & InputEvent.CTRL_DOWN_MASK) != 0);

		switch (event.getKeyCode()) {
		case KeyEvent.VK_ESCAPE:
			abacusView.shuffleDown();
			return;
		case KeyEvent.VK_UP:
			moveAbacusInput(lastX, lastY, '8', control);
			return;
		case KeyEvent.VK_LEFT:
			moveAbacusInput(lastX, lastY, '4', control);
			return;
		case KeyEvent.VK_RIGHT:
			moveAbacusInput(lastX, lastY, '6', control);
			return;
		case KeyEvent.VK_DOWN:
			moveAbacusInput(lastX, lastY, '2', control);
			return;
		default:
			processKey(event.getKeyChar(), control);
		}
	}*/

	public void keyReleased(/*FIXME KeyEvent event*/) {
		/* Method required but keyPressed does work */
	}

	public void keyTyped(/*FIXME KeyEvent event*/) {
		/* Method required but keyPressed does work */
	}
}
