package com.sillycycle.bagleyd.abacus;

import java.math.BigDecimal;

import android.app.Activity;
import android.content.Context;
//import android.content.res.Resources;
import android.media.MediaPlayer;
import android.os.Bundle;
//import android.util.Log;
import android.view.ContextMenu;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.View.OnFocusChangeListener;
import android.view.View.OnKeyListener;
import android.widget.Button;
import android.widget.EditText;

import com.sillycycle.bagleyd.abacus.view.AbacusView;

/**
 * Android UI demo program
 */
public class AbacusApp extends Activity {
	static AbacusView aView = null;
	static MediaPlayer bumpPlayer = null;
	static MediaPlayer movePlayer = null;
	static MediaPlayer dripPlayer = null;
	static final float VOLUME = (float) 0.4;
	static final boolean DEBUG = false; // noisy logs
	private static final String DISPLAY = "Display";

	/** Listen for taps. */
	private static final class TrackingTouchListener
			implements View.OnTouchListener {
		AbacusApp aApp;
		TrackingTouchListener(AbacusApp app) {
			aApp = app;
		}

		@Override public boolean onTouch(View v, MotionEvent event) {
			int action = event.getAction();

			switch (action & MotionEvent.ACTION_MASK) {
			case MotionEvent.ACTION_DOWN:
			case MotionEvent.ACTION_POINTER_DOWN:
				if (AbacusView.getAbacusCanvas() == null) {
					return false;
				}
				AbacusView.getAbacusCanvas().selectAbacus((int) event.getX(), (int) event.getY());
				break;
			case MotionEvent.ACTION_UP:
			case MotionEvent.ACTION_POINTER_UP:
			//case MotionEvent.ACTION_MOVE:
				if (AbacusView.getAbacusCanvas() == null) {
					return false;
				}
				if (AbacusView.getAbacusCanvas().releaseAbacus((int) event.getX(), (int) event.getY()) == 2) {
					final EditText editText = aApp.findViewById(R.id.text1);
					editText.setText(changeToDecimal(AbacusApp.digits));
					aApp.playMove();
					aView.invalidate();
				}
				break;
			default:
				return false;
			}
			return true;
		}
	}

	/** The application view */
	static int counter = 0;
	static String digits = "0.00";

	/** Called when the activity is first created.
	 * @param state */
	@Override public void onCreate(Bundle state) {
		super.onCreate(state);

		// install the view
		setContentView(R.layout.standard);
		//setContentView(R.layout.lee);

		// find the abacus view
		aView = findViewById(R.id.abacus);

		aView.setOnCreateContextMenuListener(this);
		aView.setOnTouchListener(new TrackingTouchListener(this));

		aView.setOnKeyListener(new OnKeyListener() {
			@Override
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if (KeyEvent.ACTION_DOWN != event.getAction()) {
					return false;
				}
				switch (keyCode) {
				case KeyEvent.KEYCODE_SPACE:
					// color = Color.MAGENTA;
					break;
				case KeyEvent.KEYCODE_ENTER:
					// color = Color.BLUE;
					break;
				default:
					return false;
				}
				//abacusCanvas.processKey((char)event.getUnicodeChar()
				//	, false /* Control*/);
				return true;
			}
		});

		// Old way: moveBeadsByPos -> ACTION_IGNORE -> updates display
		final EditText editText = findViewById(R.id.text1);
		aView.setOnFocusChangeListener(new OnFocusChangeListener() {
			@Override public void onFocusChange(View v, boolean hasFocus) {
				AbacusView.focusReact(hasFocus);
				Button buttonText = findViewById(R.id.button1);
				buttonText.setText(AbacusInterface.titleName(
						AbacusInterface.Modes.values()[counter].toString())
						+ " " + AbacusInterface.FORMAT_STRINGS[counter]);
				editText.setText(digits);
			}
		});

		// wire up the controller
		(findViewById(R.id.button1)).setOnClickListener(
				new Button.OnClickListener() {
			@Override public void onClick(View v) {
				//if (field.isSuperSaturatedResultRegister()) {
				//	resetBeads();
				//}
				changeFormatAbacus();
				String string = editText.getText().toString();
				if (string.charAt(0) == '1') {
					StringBuilder stringBuilder = new StringBuilder(string);
					stringBuilder.setCharAt(0, '0');
					string = stringBuilder.toString();
					editText.setText(changeDecimal(string));
				}
				counter++;
				if (counter >= AbacusInterface.FORMAT_STRINGS.length - 1)
					counter = 0;
				Button buttonText = findViewById(R.id.button1);
				//buttonText.setText(modeStrings[counter] + " "
				buttonText.setText(AbacusInterface.Modes.values()[counter]
					+ " " + AbacusInterface.FORMAT_STRINGS[counter]);
				playDrip();
				aView.invalidate();
			}
		});
		findViewById(R.id.menu_clear).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						clearAbacus();
					}
				});
		findViewById(R.id.menu_increment).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						incrementAbacus();
					}
				});
		findViewById(R.id.menu_undo).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						undoAbacus();
					}
				});
		findViewById(R.id.menu_complement).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						complementAbacus();
					}
				});
		findViewById(R.id.menu_decrement).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						decrementAbacus();
					}
				});
		findViewById(R.id.menu_redo).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						redoAbacus();
					}
				});


		if (aView == null)
			return;
		aView.setTextChangeListener(new AbacusView.TextChangeListener() {
			@Override public void onTextChange(String string) {
				digits = string.trim();
				editText.setText(changeToDecimal(digits));
				playBump();
				aView.invalidate();
			}
		});
	}

	/** Install an options menu.
	 * @param menu
	 * @return true */
	@Override public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.menu, menu);
		return true;
	}

	static void clearAbacus() {
		AbacusView.clearAbacus();
	}

	static void complementAbacus() {
		AbacusView.complementAbacus();
	}

	static void decrementAbacus() {
		AbacusView.decrementAbacus();
	}

	static void incrementAbacus() {
		AbacusView.incrementAbacus();
	}

	static void undoAbacus() {
		AbacusView.undoAbacus();
		aView.invalidate();
	}

	static void redoAbacus() {
		AbacusView.redoAbacus();
		aView.invalidate();
	}

	static void changeFormatAbacus() {
		AbacusView.changeFormatAbacus();
		//Log.i(DISPLAY, "changed format");
	}

	static String changeDecimal(String string) {
		// Hack to get it to work quickly... better ways TODO

		BigDecimal beads = new BigDecimal(string);
		//Log.i(DISPLAY, "beads: " + beads.toString());
		return beads.toPlainString();
	}

	static String changeToDecimal(String string) {
		// Hack to get it to work quickly... better ways TODO

		BigDecimal beads = new BigDecimal(string);
		int exponent = AbacusView.getAbacusCanvas().getDecimalPosition();
		BigDecimal hundred = BigDecimal.valueOf(Math.pow(10.0, exponent));
		BigDecimal percentage = beads.divide(hundred, exponent, BigDecimal.ROUND_HALF_UP);
		//Log.i(DISPLAY, "percentage: " + percentage.toString());
		return percentage.toPlainString();
	}

	void playBump() {
		if (DEBUG)
			return;
		Context context = aView.getContext();
		int resID = getResources().getIdentifier("bump", "raw", context.getPackageName());

		if (bumpPlayer == null) {
			bumpPlayer = MediaPlayer.create(context, resID);
			bumpPlayer.setVolume(VOLUME, VOLUME);
		} else {
			bumpPlayer.pause();
		}
		//bumpPlayer.release();
		bumpPlayer.start();
	}

	void playMove() {
		if (DEBUG)
			return;
		Context context = aView.getContext();
		int resID = getResources().getIdentifier("move", "raw", context.getPackageName());

		if (movePlayer == null) {
			movePlayer = MediaPlayer.create(context, resID);
			movePlayer.setVolume(VOLUME, VOLUME);
		} else {
			movePlayer.pause();
		}
		//movePlayer.release();
		movePlayer.start();
	}

	void playDrip() {
		if (DEBUG)
			return;
		Context context = aView.getContext();
		int resID = getResources().getIdentifier("drip", "raw", context.getPackageName());

		if (dripPlayer == null) {
			dripPlayer = MediaPlayer.create(context, resID);
			dripPlayer.setVolume(VOLUME, VOLUME);
		} else {
			dripPlayer.pause();
		}
		//dripPlayer.release();
		dripPlayer.start();
	}

	/** Respond to an options menu selection.
	 * @param item
	 * @return */
	@Override public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_clear:
			clearAbacus();
			return true;
		case R.id.menu_complement:
			complementAbacus();
			return true;
		case R.id.menu_decrement:
			decrementAbacus();
			return true;
		case R.id.menu_increment:
			incrementAbacus();
			return true;
		case R.id.menu_undo:
			undoAbacus();
			return true;
		case R.id.menu_redo:
			redoAbacus();
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	/** Install a context menu.
	 * @param menu
	 * @param v
	 * @param menuInfo */
	@Override public void onCreateContextMenu(
			ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		menu.add(Menu.NONE, 1, Menu.NONE, "Clear")
			.setAlphabeticShortcut('c');
		menu.add(Menu.NONE, 1, Menu.NONE, "Complement")
			.setAlphabeticShortcut('-');
		menu.add(Menu.NONE, 1, Menu.NONE, "Decrement")
			.setAlphabeticShortcut('d');
		menu.add(Menu.NONE, 1, Menu.NONE, "Increment")
			.setAlphabeticShortcut('i');
		menu.add(Menu.NONE, 1, Menu.NONE, "Undo")
			.setAlphabeticShortcut('u');
		menu.add(Menu.NONE, 1, Menu.NONE, "Redo")
			.setAlphabeticShortcut('r');
	}

	/** Respond to a context menu selection.
	 * @param item
	 * @return */
	@Override public boolean onContextItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case 1:
			clearAbacus();
			return true;
		case 2:
			complementAbacus();
			return true;
		case 3:
			decrementAbacus();
			return true;
		case 4:
			incrementAbacus();
			return true;
		case 5:
			undoAbacus();
			return true;
		case 6:
			redoAbacus();
			return true;
		default:
		}
		return false;
	}
}
